from .__version__ import __version__
from .api import Company, Exchange, Industry, Sector, SuperInvestor, Term

__author__ = """Lucas Montes"""
__email__ = "lluc23@hotmail.com"
__version__ = __version__
__all__ = [
    "Company",
    "Exchange",
    "Industry",
    "Sector",
    "SuperInvestor",
    "Term",
]



API_VERSION = "v1"
API_PATH = "https://inversionesyfinanzas.xyz/api/"


ALL_EXCHANGES = "lista-exchanges"
ALL_INDUSTRIES = "lista-industrias"
ALL_SECTORS = "lista-sectores"

ALL_COMPANIES = "lista-empresas"
SINGLE_COMPANY_FULL = "empresa-completa"
SINGLE_COMPANY_BASIC = "empresa-basica"

INCOME_STATEMENT = "estado-resultados"
BALANCE_SHEET = "balance-general"
CASHFLOW_STATEMENT = "estado-flujo-efectivo"

ALL_TERMS = "lista-terminos"
SINGLE_TERM = "termino"

ALL_SUPERINVESTORS = "lista-superinversores"
SINGLE_SUPERINVESTOR_HISTORIAL = "lista-historial"
SINGLE_SUPERINVESTOR_MOVEMENTS = "lista-movimientos"



from unittest.mock import patch

import pytest

from invfinsdk.api import Company, Exchange, Industry, Sector, SuperInvestor, Term


class TestExchange:
    @patch("requests.get")
    def test_get_all_exchanges(self, mock_request):
        Exchange("key").get_all_exchanges()
        mock_request.assert_called_once_with(
            url="https://inversionesyfinanzas.xyz/api/v1/lista-exchanges/",
            params={"api_key": "key"},
            headers={"User-Agent": "invfinsdk-Python"},
        )


class TestIndustry:
    @patch("requests.get")
    def test_get_all_industries(self, mock_request):
        Industry("key").get_all_industries()
        mock_request.assert_called_once_with(
            url="https://inversionesyfinanzas.xyz/api/v1/lista-industrias/",
            params={"api_key": "key"},
            headers={"User-Agent": "invfinsdk-Python"},
        )


class TestSector:
    @patch("requests.get")
    def test_get_all_sectors(self, mock_request):
        Sector("key").get_all_sectors()
        mock_request.assert_called_once_with(
            url="https://inversionesyfinanzas.xyz/api/v1/lista-sectores/",
            params={"api_key": "key"},
            headers={"User-Agent": "invfinsdk-Python"},
        )


class TestCompany:
    @patch("requests.get")
    def test_get_all_companies(self, mock_request):
        Company("key").get_all_companies()
        mock_request.assert_called_once_with(
            url="https://inversionesyfinanzas.xyz/api/v1/lista-empresas/",
            params={"api_key": "key"},
            headers={"User-Agent": "invfinsdk-Python"},
        )

    @patch("requests.get")
    def test_get_company_basic_information(self, mock_request):
        Company("key").get_company_basic_information({"slug": "AAPL"})
        mock_request.assert_called_once_with(
            url="https://inversionesyfinanzas.xyz/api/v1/empresa-basica/",
            params={"slug": "AAPL", "api_key": "key"},
            headers={"User-Agent": "invfinsdk-Python"},
        )

    @patch("requests.get")
    def test_get_company_full_information(self, mock_request):
        Company("key").get_company_full_information({"slug": "AAPL"})
        mock_request.assert_called_once_with(
            url="https://inversionesyfinanzas.xyz/api/v1/empresa-completa/",
            params={"slug": "AAPL", "api_key": "key"},
            headers={"User-Agent": "invfinsdk-Python"},
        )

    @patch("invfinsdk.api.Company.get_company_basic_information")
    @patch("invfinsdk.api.Company.get_company_full_information")
    def test_get_company(
        self,
        mock_get_company_full_information,
        mock_get_company_basic_information,
    ):
        Company("key").get_company("basic", {"param": "params"})
        mock_get_company_basic_information.assert_called_once_with({"param": "params"})
        Company("key").get_company("full", {"param2": "params2"})
        mock_get_company_full_information.assert_called_once_with({"param2": "params2"})
        with pytest.raises(Exception):
            Company("key").get_company("wrong", {"param3": "params3"})

    @patch("requests.get")
    def test_get_company_income_statements(self, mock_request):
        Company("key").get_company_income_statements({"slug": "AAPL"})
        mock_request.assert_called_once_with(
            url="https://inversionesyfinanzas.xyz/api/v1/estado-resultados/",
            params={"slug": "AAPL", "api_key": "key"},
            headers={"User-Agent": "invfinsdk-Python"},
        )

    @patch("requests.get")
    def test_get_company_balance_sheets(self, mock_request):
        Company("key").get_company_balance_sheets({"slug": "AAPL"})
        mock_request.assert_called_once_with(
            url="https://inversionesyfinanzas.xyz/api/v1/balance-general/",
            params={"slug": "AAPL", "api_key": "key"},
            headers={"User-Agent": "invfinsdk-Python"},
        )

    @patch("requests.get")
    def test_get_company_cashflow_statements(self, mock_request):
        Company("key").get_company_cashflow_statements({"slug": "AAPL"})
        mock_request.assert_called_once_with(
            url="https://inversionesyfinanzas.xyz/api/v1/estado-flujo-efectivo/",
            params={"slug": "AAPL", "api_key": "key"},
            headers={"User-Agent": "invfinsdk-Python"},
        )


class TestTerm:
    @patch("requests.get")
    def test_get_all_terms(self, mock_request):
        Term("key").get_all_terms()
        mock_request.assert_called_once_with(
            url="https://inversionesyfinanzas.xyz/api/v1/lista-terminos/",
            params={"api_key": "key"},
            headers={"User-Agent": "invfinsdk-Python"},
        )

    @patch("requests.get")
    def test_get_term(self, mock_request):
        Term("key").get_term({"id": 1})
        mock_request.assert_called_once_with(
            url="https://inversionesyfinanzas.xyz/api/v1/termino/",
            params={"id": 1, "api_key": "key"},
            headers={"User-Agent": "invfinsdk-Python"},
        )


class TestSuperInvestor:
    @patch("requests.get")
    def test_get_all_superinvestors(self, mock_request):
        SuperInvestor("key").get_all_superinvestors()
        mock_request.assert_called_once_with(
            url="https://inversionesyfinanzas.xyz/api/v1/lista-superinversores/",
            params={"api_key": "key"},
            headers={"User-Agent": "invfinsdk-Python"},
        )

    @patch("requests.get")
    def test_get_superinvestor_historial(self, mock_request):
        SuperInvestor("key").get_superinvestor_historial({"id": 1})
        mock_request.assert_called_once_with(
            url="https://inversionesyfinanzas.xyz/api/v1/lista-historial/",
            params={"id": 1, "api_key": "key"},
            headers={"User-Agent": "invfinsdk-Python"},
        )

    @patch("requests.get")
    def test_get_superinvestor_movements(self, mock_request):
        SuperInvestor("key").get_superinvestor_movements({"id": 1})
        mock_request.assert_called_once_with(
            url="https://inversionesyfinanzas.xyz/api/v1/lista-movimientos/",
            params={"id": 1, "api_key": "key"},
            headers={"User-Agent": "invfinsdk-Python"},
        )



from unittest.mock import patch

from invfinsdk.client import Client


class TestClient:
    def test_base__base_build_url(self):
        assert (
            "https://inversionesyfinanzas.xyz/api/v1/" == Client("x")._base_build_url()
        )

    def test__build_request_params(self):
        path, params = Client("x")._build_request_params("path", {})
        assert "https://inversionesyfinanzas.xyz/api/v1/path/" == path
        assert {"api_key": "x"} == params

        path, params = Client("x")._build_request_params(
            "path2", {"param": 5, "param2": "second"}
        )
        assert "https://inversionesyfinanzas.xyz/api/v1/path2/" == path
        assert {
            "api_key": "x",
            "param": 5,
            "param2": "second",
        } == params

    @patch("requests.get")
    def test__perform_request(self, mock_request):
        Client("x")._perform_request("path", {})
        mock_request.assert_called_once_with(
            url="https://inversionesyfinanzas.xyz/api/v1/path/",
            params={"api_key": "x"},
            headers={"User-Agent": "invfinsdk-Python"},
        )



#!/usr/bin/env python

"""The setup script."""

from setuptools import find_packages, setup

with open("invfinsdk/__version__.py") as version_file:
    version = version_file.read()
    version = version[-7:-2]

with open("README.md") as readme_file:
    readme = readme_file.read()

with open("HISTORY.md") as history_file:
    history = history_file.read()

requirements = [
    "requests",
]

test_requirements = [
    "pytest>=3",
]

setup(
    author="Lucas Montes",
    author_email="lluc23@hotmail.com",
    python_requires=">=3.6",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    description="An SDK to interact with InvFin endpoints",
    install_requires=requirements,
    license="MIT license",
    long_description=readme + "\n\n" + history,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords="invfinsdk",
    name="invfinsdk",
    packages=find_packages(include=["invfinsdk", "invfinsdk.*"]),
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/InvFin/Python-sdk",
    version=version,
    zip_safe=False,
)



from typing import Any, Dict, Tuple, Union

import requests

from .constants import API_PATH, API_VERSION


class Client:
    token: str

    def __init__(self, token: str) -> None:
        self.token = token

    def _base_build_url(self) -> str:
        return f"{API_PATH}{API_VERSION}/"

    def _build_request_params(
        self,
        path: str,
        params: Dict[str, Union[str, int]],
    ) -> Tuple[str, Dict[str, Union[str, int]]]:
        base_url = self._base_build_url()
        final_path = f"{base_url}{path}/"
        params.update({"api_key": self.token})
        return final_path, params

    def _perform_request(
        self,
        url: str,
        params: Dict[str, Union[str, int]],
    ) -> requests.Response:
        url, params = self._build_request_params(url, params)
        headers = {"User-Agent": "invfinsdk-Python"}
        return requests.get(url=url, params=params, headers=headers)

    def _get_clean_response(
        self,
        url: str,
        params: Dict[str, Union[str, int]] = {},
    ) -> Dict[str, Any]:
        response = self._perform_request(url, params)
        return response.json()



from typing import Any, Dict, Union

from .client import Client
from .constants import (
    ALL_COMPANIES,
    ALL_EXCHANGES,
    ALL_INDUSTRIES,
    ALL_SECTORS,
    ALL_SUPERINVESTORS,
    ALL_TERMS,
    BALANCE_SHEET,
    CASHFLOW_STATEMENT,
    INCOME_STATEMENT,
    SINGLE_COMPANY_BASIC,
    SINGLE_COMPANY_FULL,
    SINGLE_SUPERINVESTOR_HISTORIAL,
    SINGLE_SUPERINVESTOR_MOVEMENTS,
    SINGLE_TERM,
)


class Exchange(Client):
    def get_all_exchanges(self) -> Dict[str, Any]:
        return self._get_clean_response(ALL_EXCHANGES)


class Industry(Client):
    def get_all_industries(self) -> Dict[str, Any]:
        return self._get_clean_response(ALL_INDUSTRIES)


class Sector(Client):
    def get_all_sectors(self) -> Dict[str, Any]:
        return self._get_clean_response(ALL_SECTORS)


class Company(Client):
    def get_all_companies(self) -> Dict[str, Any]:
        return self._get_clean_response(ALL_COMPANIES)

    def get_company_basic_information(
        self,
        params: Dict[str, Union[str, int]],
    ) -> Dict[str, Any]:
        return self._get_clean_response(SINGLE_COMPANY_BASIC, params)

    def get_company_full_information(
        self,
        params: Dict[str, Union[str, int]],
    ) -> Dict[str, Any]:
        return self._get_clean_response(SINGLE_COMPANY_FULL, params)

    def get_company(
        self,
        amount_info: str,
        params: Dict[str, Union[str, int]],
    ) -> Dict[str, Any]:
        """Get a single company using either his id or his ticker

        Parameters
        ----------
            params : Dict[str, Union[str, int]]
                Parameters to perform a lookup
                Example: {"ticker": "INTC"}

            amount_info : str
                Either full or basic

        Returns
        -------
            Dict[str, Any]
                A company full information
        """
        if amount_info == "basic":
            return self.get_company_basic_information(params)
        elif amount_info == "full":
            return self.get_company_full_information(params)
        raise Exception("You must set either full or basic")

    def get_company_income_statements(
        self,
        params: Dict[str, Union[str, int]],
    ) -> Dict[str, Any]:
        return self._get_clean_response(INCOME_STATEMENT, params)

    def get_company_balance_sheets(
        self,
        params: Dict[str, Union[str, int]],
    ) -> Dict[str, Any]:
        return self._get_clean_response(BALANCE_SHEET, params)

    def get_company_cashflow_statements(
        self,
        params: Dict[str, Union[str, int]],
    ) -> Dict[str, Any]:
        return self._get_clean_response(CASHFLOW_STATEMENT, params)


class Term(Client):
    def get_all_terms(self) -> Dict[str, Any]:
        return self._get_clean_response(ALL_TERMS)

    def get_term(self, params: Dict[str, Union[str, int]]) -> Dict[str, Any]:
        """Get a single term using either his id or his slug

        Parameters
        ----------
            params : Dict[str, Union[str, int]]
                Parameters to perform a lookup
                Example: {"slug": "precio-valor-en-libros"}

        Returns
        -------
            Dict[str, Any]
                A full term
        """
        return self._get_clean_response(SINGLE_TERM, params)


class SuperInvestor(Client):
    def get_all_superinvestors(self) -> Dict[str, Any]:
        return self._get_clean_response(ALL_SUPERINVESTORS)

    def get_superinvestor_historial(
        self,
        params: Dict[str, Union[str, int]],
    ) -> Dict[str, Any]:
        """Get all the historial of a superinvestor

        Parameters
        ----------
            params : Dict[str, Union[str, int]]
                Parameters to perform a lookup
                Example: {"id": 45}

        Returns
        -------
            Dict[str, Any]
                All the superinvestor historial
        """
        return self._get_clean_response(SINGLE_SUPERINVESTOR_HISTORIAL, params)

    def get_superinvestor_movements(
        self,
        params: Dict[str, Union[str, int]],
    ) -> Dict[str, Any]:
        """Get all the movements of a superinvestor

        Parameters
        ----------
            params : Dict[str, Union[str, int]]
                Parameters to perform a lookup
                Example: {"slug": "precio-valor-en-libros"}

        Returns
        -------
            Dict[str, Any]
                All the superinvestor movements
        """
        return self._get_clean_response(SINGLE_SUPERINVESTOR_MOVEMENTS, params)
